{design=devteams.at/conference;author=@dtanzer;slideNumbers=true}

---{background=./bike.jpg;slide-design=blank}

^ Something not right: Sports bike, but...
^ 
^ * mud guards
^ * lock
^ * kick stand
^ * ...

#{slide-design=highlight; break=always } 1. Tradeoffs Everywhere

^ and there's always context to consider

---

*{reveal=next} Inside-Out, **no mocks**: Chicago School TDD
* Outside-in, **embrace mocks**: London School TDD
* Outside-in, **no mocks**: Munich School TDD

^ You read a blog post called "Mocks are Evil"...  
^ Schools of TDD  
^ Should use mocks?  
^ Robert C. Martin **nephew**, Nat Price, David Völkel

#{slide-design=title; break=always} To Mock or Not to Mock

---{background=./flex.jpg;slide-design=blank}

^ A powerfuld but dangerous tool

---{background=./flex-cable.jpg;slide-design=blank}

^ Use only when benefits outweigh the risks

---{background=./flex-gear.jpg;slide-design=blank}

^ Take some protective measures

#{slide-design=highlight; break=always } 2. Powerful, Dangerous Tool

---

```markdown
{ foo; bar=baz }
```

```typescript
it('returns option for correctly parsed option', () => {
	const option = 'foo = bar'

	const result = optionParser.parse(option, 0, option.length)

	expect(result).toHaveProperty('startIndex', 0)
	expect(result).toHaveProperty('length', option.length)
	expect(result?.content).toHaveProperty('key', 'foo')
	expect(result?.content).toHaveProperty('value', 'bar')
	expect(result?.content).toHaveProperty('asText', 'foo = bar')
})
```

^ Simple test, not mocks, no dependencies  
^ Inside-Out, Chicago School  
^ Quick aside: Universal Architecture and Inside-Out TDD

---

![Universal Architecture](./universal_architecutre.jpg)

_[Andrea Angealla: Universal Architecture (Coined by J.B. Rainsberger)](https://www.productivecsharp.com/2017/03/universal-architecture/)_

^ * Generalization **-and-**
^ * Simplification of
^ Hexagonal Architecture (Ports & Adapters)
^ 
^ Useful model for discussion **because** it is so simple

---{background=./happy_zone_explained.jpg}
---{background=./dmz_explained.jpg}
^ Protects the happy zone.  
^ The messy integration code

---{background=./outside_world_examples.jpg}
---{background=./outside_world_monsters.jpg}

---{background=./happy_zone_arrows.jpg}
^ No outward dependencies from the happy zone  
^ DMZ knows both

---{background=./inside_out_1.jpg}

---{background=./inside_out_2.jpg}

---

```markdown
{ foo; bar=baz }
```

```typescript
	it('parses key-value option at start of the options block', () => {
		const options = '{ foo = bar}'

		const result = parse(options)

		expect(result?.content.asMap).toHaveProperty('foo', 'bar')
		expect(result?.length).toEqual(options.length)
	})
```

^ Scenario: Inside-out, no mocks, depending on "Option"

# But!

```typescript
let nextParser = this.=>{ top-right; reveal=next}defaultOptionParser
let nextOption: ParserResult<Option> | null
do {
    nextOption = =>{reveal=next}nextParser.parse(text, start+i, length-i)
    if(nextOption) {
        // ...
    }
} while(nextOption != null)
```

#{slide-design=highlight; break=always } 3. No Mocks: Error Boundary

^ Makes **defect-localization** harder

# Test Doubles

*{reveal=next} **Dummy**: Just a value
* **Stub**: Returns canned answers
* **Fake**: Simpler implementation
* **Mock**: Has interaction-verification baked in
* **Spy**: Can be asked questions about interactions later

_[Martin Fowler: Mocks aren't Stubs](https://martinfowler.com/articles/mocksArentStubs.html#TheDifferenceBetweenMocksAndStubs)_

# Dummy: Just Make it Run!

---
```typescript
const locations: [string, any][] = [
	[ 'left', RightArrow, ],
    //...
]
locations.forEach(td => {
	it(`creates a ${td[1]} when direction is ${td[0]}`, () => {
		const arrowRenderer = mount(
            <ArrowRenderer location={td[0]}>=>{reveal=next}text</ArrowRenderer>
        )

		expect(arrowRenderer.find(td[1])).to.have.length(1)
	})
})
```

^ In this test, there's a **dummy value** hidden somewhere!

---
```typescript
const locations: [string, any][] = [
	[ 'left', RightArrow, ],
    //...
]
locations.forEach(td => {
	it(`creates a ${td[1]} when direction is ${td[0]}`, () => {
		const arrowRenderer = mount(
            <ArrowRenderer location={td[0]}>=>{}dummy</ArrowRenderer>
        )

		expect(arrowRenderer.find(td[1])).to.have.length(1)
	})
})
```

#{slide-design=highlight; break=always } 4. Make Test-Doubles Visible

---

```typescript
const createPresentationWindow = (
    overrideProperties: Partial<PresentationWindowProperties>  = {}
) => {
	const defaultProperties: Required<PresentationWindowProperties>={
		_getVersion: () => 'dummy',
        //...
	}
	const properties = { ...defaultProperties, ...overrideProperties, }

	const result = mount(<PresentationWindow
		{...properties}
	/>)
	result.update()
	return result
}
```

# Stub: Answer-in-a-Can

```typescript
it('displays the version number', () => {
	const presentationWindow = createPresentationWindow({
        _getVersion: () => '1.2.3',
    })

	expect(/*...*/).toHaveTextContent('1.2.3')
})
```

---{background=./stub_happy_zone.jpg}

#{break=always} Fake: Something, but not Real

---

```typescript
const createPresentationWindow = (
    overrideProperties: Partial<PresentationWindowProperties>  = {}
) => {
	const defaultProperties: Required<PresentationWindowProperties>={
		_setTimeout: (cb: ()=>unknown) => { cb() },
        //...
	}
	const properties = { ...defaultProperties, ...overrideProperties, }

	const result = mount(<PresentationWindow
		{...properties}
	/>)
	result.update()
	return result
}
```

---{background=./fake_dmz.jpg}

#{break=always;background=./sequence_set_presentation_data.jpg} Setting Presentation Data


---

```typescript
const onSetPresentationData = (callback: OnSetPresentationDataCB) => {
	const document = parseMarkdown('# first\n# second\n# third\n')
	callback(document, 'design', 1, 'basePath',
        new Annotations().serialize())
}
```

```typescript
const createPresentationWindow = (
    overrideProperties: Partial<PresentationWindowProperties>  = {}
) => {
	const defaultProperties: Required<PresentationWindowProperties>={
		_onSetPresentationData: onSetPresentationData,
        //...
	}
    //...
}
```

#{slide-design=highlight; break=always } 5. Fake DMZ / Outside World

#{break=always} Mock: Use Correctly or Blow Up

---
```typescript
it('sends start-video event when start button was clicked', () => {
	const startVideoMock = mock<(id: string)=>void>('startVideoMock')
	=>{reveal=next}when(startVideoMock(anyString())).return().once()=>{top-right;reveal=next} 
	const { controls } = startConfiguration({
        _startVideo: instance(startVideoMock),
    })

	act(() => { 
        controls.find('.start-mirroring').at(0).simulate('click')
    })
	controls.update()

	=>{reveal=next}verify(startVideoMock)
})
```

^ Arrows!

---

## Result-Based

* Observe result or state
* Verify result matches the expected result
* Use dummies, stubs or fakes _if you need them_
* `expect`
* "Frontdoor testing"

## Interaction-Based

* Nothing to observe
* Verify expected interaction has happened
* Use mocks and/or spies; and dummies, stubs or fakes _if you need them_
* `when` / `verify`
* Backdoor testing

#{slide-design=highlight; break=always } 6. Result -or- Interaction

---{background=./mock_dmz.jpg}

---{background=./mock_happy_zone.jpg;break=always}

^ Mocks and Outside-In

# Special Case: Jest Mock ES-Module


```javascript
"^(.*)/MonacoEditor$": "<rootDir>/__mocks__/reactMonacoMock.js",
```

```javascript
module.exports = {
	languages: {
		register: function(language) {},
		setMonarchTokensProvider: function(name, tokens) {},
		registerCompletionItemProvider: function(name, provider) {}
	},
	editor: {
		defineTheme: function(name, theme) {}
	}
};
```

# Mock / Fake Close to DMZ

```typescript
it('loads 2 designs when 2 designs are found', () => {
	const findSlideDesignsMock = 
        mock<(p: string, d: SlideDesign[])=>void>('findDesignsMock')
	when(findSlideDesignsMock(anyString(), anyArray()))
        .callFake((p, d) => {
    		d.push({ /* ... */ })
    		d.push({ /* ... */ })
    	})

	const designs = loadAllDesigns(undefined, instance(findSlideDesignsMock))

	expect(loadedDesigns).to.have.length(2)
	expect(loadedDesigns[0]).to.contain({ id: 'design/1', })
	expect(loadedDesigns[1]).to.contain({ id: 'design/2', })
})
```

---

```typescript
it('clones the design when everything is OK', () => {
	const cloneDesignFolderMock = 
        mock<(
            id: string, basePath: string | undefined, 
            pathToClone: string
        ) => string | void>('cloneDesignMock')
	when(cloneDesignFolderMock(
        'foo/bar', '/base/path', 'path/to/clone'
    )).return().once()

	cloneDesign('foo/bar', '/base/path', 'path/to/clone', 
        () => [], instance(cloneDesignFolderMock))

	verify(cloneDesignFolderMock)
})
```

#{slide-design=highlight; break=always } 7. Don't Mock what You don't Own

---{background=./integration_test_file.jpg}

^ Scenario: Interacting with outside world
^ Do not mock what you don't own - Integration Test it!

---

```typescript
it('finds existing designs', () => {
	const designs: SlideDesign[] = []
	findSlideDesigns(path.join(__dirname, 'designs'), designs)
	expect(designs).to.have.length(2)
})
```

```
 |- findSlideDesigns.spec.ts
 |- designs
 |    \- marmota
 |        |- d1
 |        |    |- design.json
 |        |    |- design.css
 |        |    \- preview.md
 |        |- d2
 ...
```

#{slide-design=highlight; break=always } 8. Integration-Test at Boundary

# Boundary AND Hard to Test

## Electron IPC

```typescript
sendSync(channel: C_MAIN, ...args: any[]) {
	return ipcRenderer.sendSync(channel, ...args)
}
```

```typescript
sendSync(window: BrowserWindow, channel: C_RENDERER, ...args: any[]) {
	window.webContents.send(channel, ...args)
}
```

^ Hard to test: Multiple processes, ...

---

```typescript
export type ChannelToMain = {
	'--annotation--delete-annotation':
        (fromWindow: AnnotationWindowName, annotationId: string)
            => unknown,
}
```

```typescript
sendSync<C extends keyof C_MAIN>(
    channel: C, ...args: InferArguments<C_MAIN[C]>
): InferReturnType<C_MAIN[C]> {
	if(typeof channel !== 'string') throw new Error('...')
	return ipcRenderer.sendSync(channel, ...args)
}
```

---

```typescript
export interface AnnotationsApi {
	deleteAnnotation: ChannelToMain['--annotation--delete-annotation']
}
```

```typescript
export const annotations: AnnotationsApi = {
	deleteAnnotation:
        (fromWindow: AnnotationWindowName, annotationId: string) => 
            rendererIpc.sendSync(
                '--annotation--delete-annotation',
                fromWindow, annotationId
            )
}
```

---

```typescript
on(callbacks: { [key in keyof C_MAIN]: C_MAIN[key]}) {
	Object.keys(callbacks).forEach((channel) => {
		if(typeof channel !== 'string') throw new Error('...')
		ipcMain.on(channel, (ipcEvent, ...args) => {
			ipcEvent.returnValue=callbacks[channel](...args, ipcEvent)
		})
	})
}
```

```typescript
'--annotation--delete-annotation': 
    (fromWindow: AnnotationWindowName, annotationId: string) => {
	    // ...
    },
```

#{slide-design=highlight; break=always } 9. Leverage the Type System

#{slide-design=subtitle} To Recap...

---

1.{reveal=next} Tradeoffs Everywhere
2. Powerful, Dangerous Tool
3. No Mocks: Error Boundary
4. Make Test-Doubles Visible
5. Fake DMZ / Outside World
6. Result- -or- Interaction-based
7. Don't Mock what You don't Own
8. Integration-Test at Boundary
9. Leverage the Type System

# Heuristics: Mocks

*{reveal=next} Do not mock what you don't own
* Avoid mocks to make untestable code testable
* Avoid mocks that return mocks
* Avoid `expect` and `verify` in the same test
    * Don't mix result-based and interaction-based testing

---{slide-design=about-author;background=./david-tanzer-2019-portrait.jpg;background-position=center,right;background-width=45}

## About David

* Trainer, Coach, Developer
* [https://davidtanzer.net](https://davidtanzer.net)
* [@dtanzer](https://twitter.com/dtanzer)

Slides:  
[marmota.app/blog/talk_mocks/](https://marmota.app/blog/talk_mocks/)  
[speakerdeck.com/dtanzer/](https://speakerdeck.com/dtanzer/to-mock-or-not-to-mock)
